package com.sintecmedia.components.renderkit;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.javascript.JSFunctionDefinition;
import org.ajax4jsf.javascript.JSLiteral;
import org.ajax4jsf.renderkit.AjaxComponentRendererBase;
import org.ajax4jsf.renderkit.AjaxRendererUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sintecmedia.components.component.UIWeekDaysButtons;
import com.sintecmedia.components.event.DaySelectedEvent;

public abstract class WeekDaysButtonsRendererBase 
		extends AjaxComponentRendererBase {

	private static final Log _log = 
		LogFactory.getLog( WeekDaysButtonsRendererBase.class );
	
	protected Class getComponentClass() {
		return UIWeekDaysButtons.class;
	}
	 
	protected void doDecode(FacesContext context, UIComponent component) {
		Map paramMap = getParamMap(context);
    	String clientId = component.getClientId(context);
    	String param = (String) paramMap.get(clientId);        
        if (param == null) {
        	return;
        }
                
		UIWeekDaysButtons dayButtons = (UIWeekDaysButtons) component;
		int day = dayButtons.getDayIndex();
		if( day >= 0 ) {
			DaySelectedEvent event =
				new DaySelectedEvent(			
						dayButtons, String.valueOf(day), param, day );
		     event.queue();
		}       
	}

	private Map getParamMap(FacesContext context) {
        return context.getExternalContext().getRequestParameterMap();
    }

	public String getOnClick(
			FacesContext context, UIComponent component, Integer dayIndex ) {
		return 
			"Event.fire(this, 'rich:weekdaysbuttons:ondayselect', {'dayIndex': '" + dayIndex + "'});";	
	}

	public Integer getIndexByDay( UIComponent component, Object dayName ) {
		return
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH 
				).get( dayName.toString() );
	}
	
	public String getSubmitFunction(
			FacesContext context, UIComponent component ) {
		JSFunctionDefinition definition = new JSFunctionDefinition("event");
	        
		JSFunction function = 
			AjaxRendererUtils.buildAjaxFunction( component, context );
        Map eventOptions = 
        	AjaxRendererUtils.buildEventOptions( context, component, true );
        Map parameters = (Map) eventOptions.get( "parameters" );
        
        Map params = getParameters( context,component );
        if(!params.isEmpty()){
        	parameters.putAll(params);
        } 
        
        parameters.put(
        		component.getClientId(context), 
        		new JSLiteral("event.memo.dayIndex") );        
        function.addParameter(eventOptions);
        StringBuffer buffer = new StringBuffer();
        function.appendScript(buffer);
        buffer.append("; return false;");
		definition.addToBody(buffer.toString());			        
		return definition.toScript();
	}
	
	//get UIParameter's Map
    protected Map getParameters(FacesContext context, UIComponent component){
    	Map parameters = new HashMap(); 
    	
    	if( component instanceof UIWeekDaysButtons ){
    		UIWeekDaysButtons weekDaysButtons = (UIWeekDaysButtons)component;
    		List children = weekDaysButtons.getChildren();
    		for (Iterator iterator = children.iterator(); iterator.hasNext();) {
				UIComponent child = (UIComponent) iterator.next();
				if(child instanceof UIParameter) {
					UIParameter param = (UIParameter)child;
					String name = param.getName();
					if (name != null) {
						parameters.put(name, param.getValue());
					}
				}
			}
    	}
    	    	
    	return parameters;
    }
    
	/**
	 * 
	 * @param component UIWeeklyCalendar component used.
	 * @return list of days from component data model as it called in default java.util.Calendar intance
	 */
	public List<String> getDaysOfWeek( UIComponent component ) {
		//Days map returned from calendar relays on day's name as key, so it
		//needs refinement by day's index.
		Map<String, Integer> daysMap = 
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH );
		String[] names = new String[ daysMap.size() ]; 
		for( String name : daysMap.keySet() ) {
			names[ daysMap.get( name ) - 1 ] = name;
		}			
		
		List<String> result = new ArrayList<String>( 7 );
		for( int i = 0; i < 7; i++ ) {			
			result.add( names[i] );
		}
		return result;
	}
}
